#!/opt/alt/python37/bin/python3 -bb
# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2020 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#

# pylint: disable=no-absolute-import

import getopt
import os
import platform
import shutil
import sys

from sentry import init_wmt_sentry_client, setup_logger

logger = setup_logger('daemon_control')

DAEMON_NAME: str = 'cl_wmt_scanner'
_CL_PLUS_DIR: str = '/usr/share/web-monitoring-tool/service'
# CL6 service files
_CL6_SERVICE_FILE_PATH: str = '/etc/rc.d/init.d'
_CL6_SERVICE_WMT_FILE: str = 'cl_wmt_scanner'
# CL7/8 service files
_CL7_8_SYSTEMD_UNIT_FILE_PATH: str = '/etc/systemd/system'
_CL7_8_SYSTEMD_WMT_UNIT_FILE: str = 'cl_wmt_scanner.service'

# Utilities
_SERVICE_BIN: str = '/sbin/service'
_CHKCONFIG_BIN: str = '/sbin/chkconfig'
_SYSTEMCTL_BIN: str = '/usr/bin/systemctl'


def usage():
    print('')
    print('Use following syntax to manage LVE Manager install utility:')
    print(sys.argv[0] + " [OPTIONS]")
    print('Options:')
    print(" -i | --install     : install web monitoring tool service")
    print(" -d | --delete      : delete  web monitoring tool service")


def install_wmt_daemon():
    """
    Install CL wmt daemon
    :return: None
    """
    _is_cl6 = 'el6' in platform.release()
    if _is_cl6:
        # CL6
        # /usr/share/cloudlinux/cl_plus/service/cl_wmt_scanner -> /etc/rc.d/init.d/cl_wmt_scanner
        src = os.path.join(_CL_PLUS_DIR, _CL6_SERVICE_WMT_FILE)
        dest = os.path.join(_CL6_SERVICE_FILE_PATH, _CL6_SERVICE_WMT_FILE)
        shutil.copy(src, dest)
        os.chmod(dest, 0o755)
    else:
        # CL7, CL8
        # /usr/share/cloudlinux/cl_plus/service/cl_wmt_scanner.service -> /etc/systemd/system/cl_wmt_scanner.service
        src = os.path.join(_CL_PLUS_DIR, _CL7_8_SYSTEMD_WMT_UNIT_FILE)
        dest = os.path.join(_CL7_8_SYSTEMD_UNIT_FILE_PATH, _CL7_8_SYSTEMD_WMT_UNIT_FILE)
        shutil.copy(src, dest)
        os.chmod(dest, 0o644)
        os.system('{} daemon-reload'.format(_SYSTEMCTL_BIN))


def enable_wmt_daemon():
    """
    Enable cl_wmt_scanner service
    """
    _is_cl6 = 'el6' in platform.release()
    if _is_cl6:
        # CL6
        # Start wmt service
        os.system('{} {} start >/dev/nul 2>&1'.format(_SERVICE_BIN, DAEMON_NAME))
        os.system('{} --add {} >/dev/nul 2>&1'.format(_CHKCONFIG_BIN, DAEMON_NAME))
    else:
        # CL7, CL8
        os.system('{} daemon-reload'.format(_SYSTEMCTL_BIN))
        # Start wmt daemon
        os.system('{} enable {}'.format(_SYSTEMCTL_BIN, DAEMON_NAME))
        os.system('{} start {}'.format(_SYSTEMCTL_BIN, DAEMON_NAME))


def disable_wmt_daemon():
    """
    Disable WMT daemon
    :return:
    """
    _is_cl6 = 'el6' in platform.release()
    if _is_cl6:
        # CL6
        os.system('{} {} stop >/dev/nul 2>&1'.format(_SERVICE_BIN, DAEMON_NAME))
        os.system('{} --del {} >/dev/nul 2>&1'.format(_CHKCONFIG_BIN, DAEMON_NAME))
    else:
        # CL7, CL8
        os.system('{} kill cl_wmt_scanner >/dev/nul 2>&1'.format(_SYSTEMCTL_BIN))
        os.system('{} disable cl_wmt_scanner >/dev/nul 2>&1'.format(_SYSTEMCTL_BIN))


def stop_remove_wmt_daemon():
    """
    Remove WMT daemon from system
    :return:
    """
    disable_wmt_daemon()
    _is_cl6 = 'el6' in platform.release()
    if _is_cl6:
        # CL6
        try:
            # Remove /etc/rc.d/init.d/cl_plus_sender
            src = os.path.join(_CL6_SERVICE_FILE_PATH, _CL6_SERVICE_WMT_FILE)
            os.remove(src)
        except (OSError, IOError):
            pass
    else:
        # CL7, CL8
        try:
            # Remove /etc/systemd/system/cl_plus_sender.service
            src = os.path.join(_CL7_8_SYSTEMD_UNIT_FILE_PATH, _CL7_8_SYSTEMD_WMT_UNIT_FILE)
            os.remove(src)
        except (OSError, IOError):
            pass
        os.system('{} daemon-reload >/dev/nul 2>&1'.format(_SYSTEMCTL_BIN))


if __name__ == "__main__":
    init_wmt_sentry_client()
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hid", ["help", "install", "delete"])
    except getopt.GetoptError as err:
        # print help information and exit:
        print(str(err))  # will print something like "option -a not recognized"
        usage()
        sys.exit(2)

    try:
        for o, a in opts:
            if o in ("-h", "--help"):
                usage()
                sys.exit()
            elif o in ("-i", "--install"):
                install_wmt_daemon()
            elif o in ("-d", "--delete"):
                stop_remove_wmt_daemon()
            else:
                usage()
                sys.exit(2)
    except Exception as e:
        logger.exception(e)
